@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 """html2text: Turn HTML into equivalent Markdown-structured text."""
-__version__ = "3.101"
+__version__ = "3.200.3"
 __author__ = "Aaron Swartz (me@aaronsw.com)"
 __copyright__ = "(C) 2004-2008 Aaron Swartz. GNU GPL 3."
 __contributors__ = ["Martin 'Joey' Schulze", "Ricardo Reyes", "Kevin Jay North"]
@@ -56,6 +56,7 @@ GOOGLE_LIST_INDENT = 36
 
 IGNORE_ANCHORS = False
 IGNORE_IMAGES = False
+IGNORE_EMPHASIS = False
 
 ### Entity Nonsense ###
 
@@ -68,13 +69,13 @@ def name2cp(k):
         if k.startswith("&#") and k.endswith(";"): return int(k[2:-1]) # not in latin-1
         return ord(codecs.latin_1_decode(k)[0])
 
-unifiable = {'rsquo':"'", 'lsquo':"'", 'rdquo':'"', 'ldquo':'"', 
+unifiable = {'rsquo':"'", 'lsquo':"'", 'rdquo':'"', 'ldquo':'"',
 'copy':'(C)', 'mdash':'--', 'nbsp':' ', 'rarr':'->', 'larr':'<-', 'middot':'*',
 'ndash':'-', 'oelig':'oe', 'aelig':'ae',
-'agrave':'a', 'aacute':'a', 'acirc':'a', 'atilde':'a', 'auml':'a', 'aring':'a', 
-'egrave':'e', 'eacute':'e', 'ecirc':'e', 'euml':'e', 
+'agrave':'a', 'aacute':'a', 'acirc':'a', 'atilde':'a', 'auml':'a', 'aring':'a',
+'egrave':'e', 'eacute':'e', 'ecirc':'e', 'euml':'e',
 'igrave':'i', 'iacute':'i', 'icirc':'i', 'iuml':'i',
-'ograve':'o', 'oacute':'o', 'ocirc':'o', 'otilde':'o', 'ouml':'o', 
+'ograve':'o', 'oacute':'o', 'ocirc':'o', 'otilde':'o', 'ouml':'o',
 'ugrave':'u', 'uacute':'u', 'ucirc':'u', 'uuml':'u',
 'lrm':'', 'rlm':''}
 
@@ -83,42 +84,6 @@ unifiable_n = {}
 for k in unifiable.keys():
     unifiable_n[name2cp(k)] = unifiable[k]
 
-def charref(name):
-    if name[0] in ['x','X']:
-        c = int(name[1:], 16)
-    else:
-        c = int(name)
-    
-    if not UNICODE_SNOB and c in unifiable_n.keys():
-        return unifiable_n[c]
-    else:
-        try:
-            return unichr(c)
-        except NameError: #Python3
-            return chr(c)
-
-def entityref(c):
-    if not UNICODE_SNOB and c in unifiable.keys():
-        return unifiable[c]
-    else:
-        try: name2cp(c)
-        except KeyError: return "&" + c + ';'
-        else:
-            try:
-                return unichr(name2cp(c))
-            except NameError: #Python3
-                return chr(name2cp(c))
-
-def replaceEntities(s):
-    s = s.group(1)
-    if s[0] == "#": 
-        return charref(s[1:])
-    else: return entityref(s)
-
-r_unescape = re.compile(r"&(#?[xX]?(?:[0-9a-fA-F]+|\w{1,8}));")
-def unescape(s):
-    return r_unescape.sub(replaceEntities, s)
-
 ### End Entity Nonsense ###
 
 def onlywhite(line):
@@ -128,31 +93,6 @@ def onlywhite(line):
             return c is ' '
     return line
 
-def optwrap(text):
-    """Wrap all paragraphs in the provided text."""
-    if not BODY_WIDTH:
-        return text
-    
-    assert wrap, "Requires Python 2.3."
-    result = ''
-    newlines = 0
-    for para in text.split("\n"):
-        if len(para) > 0:
-            if para[0] != ' ' and para[0] != '-' and para[0] != '*':
-                for line in wrap(para, BODY_WIDTH):
-                    result += line + "\n"
-                result += "\n"
-                newlines = 2
-            else:
-                if not onlywhite(para):
-                    result += para + "\n"
-                    newlines = 1
-        else:
-            if newlines < 2:
-                result += "\n"
-                newlines += 1
-    return result
-
 def hn(tag):
     if tag[0] == 'h' and len(tag) == 2:
         try:
@@ -174,7 +114,10 @@ def dumb_css_parser(data):
 
     # parse the css. reverted from dictionary compehension in order to support older pythons
     elements =  [x.split('{') for x in data.split('}') if '{' in x.strip()]
-    elements = dict([(a.strip(), dumb_property_dict(b)) for a, b in elements])
+    try:
+        elements = dict([(a.strip(), dumb_property_dict(b)) for a, b in elements])
+    except ValueError:
+        elements = {} # not that important
 
     return elements
 
@@ -198,13 +141,6 @@ def google_list_style(style):
             return 'ul'
     return 'ol'
 
-def google_nest_count(style):
-    """calculate the nesting count of google doc lists"""
-    nest_count = 0
-    if 'margin-left' in style:
-        nest_count = int(style['margin-left'][:-2]) / GOOGLE_LIST_INDENT
-    return nest_count
-
 def google_has_height(style):
     """check if the style of the element has the 'height' attribute explicitly defined"""
     if 'height' in style:
@@ -238,13 +174,26 @@ def list_numbering_start(attrs):
     else:
         return 0
 
-class _html2text(HTMLParser.HTMLParser):
+class HTML2Text(HTMLParser.HTMLParser):
     def __init__(self, out=None, baseurl=''):
         HTMLParser.HTMLParser.__init__(self)
-        
+
+        # Config options
+        self.unicode_snob = UNICODE_SNOB
+        self.links_each_paragraph = LINKS_EACH_PARAGRAPH
+        self.body_width = BODY_WIDTH
+        self.skip_internal_links = SKIP_INTERNAL_LINKS
+        self.inline_links = INLINE_LINKS
+        self.google_list_indent = GOOGLE_LIST_INDENT
+        self.ignore_links = IGNORE_ANCHORS
+        self.ignore_images = IGNORE_IMAGES
+        self.ignore_emphasis = IGNORE_EMPHASIS
+        self.google_doc = False
+        self.ul_item_mark = '*'
+
         if out is None: self.out = self.outtextf
         else: self.out = out
-        self.outtextlist = [] # empty list to store output characters before they are  "joined"
+        self.outtextlist = [] # empty list to store output characters before they are "joined"
         try:
             self.outtext = unicode()
         except NameError: # Python3
@@ -276,52 +225,62 @@ class _html2text(HTMLParser.HTMLParser):
         self.abbr_list = {} # stack of abbreviations to write later
         self.baseurl = baseurl
 
-        if options.google_doc:
-            del unifiable_n[name2cp('nbsp')]
-            unifiable['nbsp'] = '&nbsp_place_holder;'
-    
-    def outtextf(self, s): 
+        try: del unifiable_n[name2cp('nbsp')]
+        except KeyError: pass
+        unifiable['nbsp'] = '&nbsp_place_holder;'
+
+
+    def feed(self, data):
+        data = data.replace("</' + 'script>", "</ignore>")
+        HTMLParser.HTMLParser.feed(self, data)
+
+    def handle(self, data):
+        self.feed(data)
+        self.feed("")
+        return self.optwrap(self.close())
+
+    def outtextf(self, s):
         self.outtextlist.append(s)
         if s: self.lastWasNL = s[-1] == '\n'
-    
+
     def close(self):
         HTMLParser.HTMLParser.close(self)
-        
+
         self.pbr()
         self.o('', 0, 'end')
 
         self.outtext = self.outtext.join(self.outtextlist)
-        
-        if options.google_doc:
+
+        if self.google_doc:
             self.outtext = self.outtext.replace('&nbsp_place_holder;', ' ');
-        
+
         return self.outtext
-        
+
     def handle_charref(self, c):
-        self.o(charref(c), 1)
+        self.o(self.charref(c), 1)
 
     def handle_entityref(self, c):
-        self.o(entityref(c), 1)
-            
+        self.o(self.entityref(c), 1)
+
     def handle_starttag(self, tag, attrs):
         self.handle_tag(tag, attrs, 1)
-    
+
     def handle_endtag(self, tag):
         self.handle_tag(tag, None, 0)
-        
+
     def previousIndex(self, attrs):
         """ returns the index of certain set of attributes (of a link) in the
             self.a list
- 
+
             If the set of attributes is not found, returns None
         """
         if not has_key(attrs, 'href'): return None
-        
+
         i = -1
         for a in self.a:
             i += 1
             match = 0
-            
+
             if has_key(a, 'href') and a['href'] == attrs['href']:
                 if has_key(a, 'title') or has_key(attrs, 'title'):
                         if (has_key(a, 'title') and has_key(attrs, 'title') and
@@ -335,14 +294,14 @@ class _html2text(HTMLParser.HTMLParser):
     def drop_last(self, nLetters):
         if not self.quiet:
             self.outtext = self.outtext[:-nLetters]
-           
+
     def handle_emphasis(self, start, tag_style, parent_style):
         """handles various text emphases"""
         tag_emphasis = google_text_emphasis(tag_style)
         parent_emphasis = google_text_emphasis(parent_style)
 
         # handle Google's text emphasis
-        strikethrough =  'line-through' in tag_emphasis and options.hide_strikethrough
+        strikethrough =  'line-through' in tag_emphasis and self.hide_strikethrough
         bold = 'bold' in tag_emphasis and not 'bold' in parent_emphasis
         italic = 'italic' in tag_emphasis and not 'italic' in parent_emphasis
         fixed = google_fixed_width_font(tag_style) and not \
@@ -406,7 +365,7 @@ class _html2text(HTMLParser.HTMLParser):
         else:
             attrs = dict(attrs)
 
-        if options.google_doc:
+        if self.google_doc:
             # the attrs parameter is empty for a closing tag. in addition, we
             # need the attributes of the parent nodes in order to get a
             # complete style description for the current element. we assume
@@ -432,14 +391,14 @@ class _html2text(HTMLParser.HTMLParser):
                 return # prevent redundant emphasis marks on headers
 
         if tag in ['p', 'div']:
-            if options.google_doc:
+            if self.google_doc:
                 if start and google_has_height(tag_style):
                     self.p()
                 else:
                     self.soft_br()
             else:
                 self.p()
-        
+
         if tag == "br" and start: self.o("  \n")
 
         if tag == "hr" and start:
@@ -447,7 +406,7 @@ class _html2text(HTMLParser.HTMLParser):
             self.o("* * *")
             self.p()
 
-        if tag in ["head", "style", 'script']: 
+        if tag in ["head", "style", 'script']:
             if start: self.quiet += 1
             else: self.quiet -= 1
 
@@ -457,29 +416,29 @@ class _html2text(HTMLParser.HTMLParser):
 
         if tag in ["body"]:
             self.quiet = 0 # sites like 9rules.com never close <head>
-        
+
         if tag == "blockquote":
-            if start: 
+            if start:
                 self.p(); self.o('> ', 0, 1); self.start = 1
                 self.blockquote += 1
             else:
                 self.blockquote -= 1
                 self.p()
         
-        if tag in ['em', 'i', 'u']: self.o("_")
-        if tag in ['strong', 'b']: self.o("**")
+        if tag in ['em', 'i', 'u'] and not self.ignore_emphasis: self.o("_")
+        if tag in ['strong', 'b'] and not self.ignore_emphasis: self.o("**")
         if tag in ['del', 'strike']:
-            if start:                                                           
+            if start:
                 self.o("<"+tag+">")
             else:
                 self.o("</"+tag+">")
 
-        if options.google_doc:
+        if self.google_doc:
             if not self.inheader:
                 # handle some font attributes, but leave headers clean
                 self.handle_emphasis(start, tag_style, parent_style)
 
-        if tag == "code" and not self.pre: self.o('`') #TODO: `` `this` ``
+        if tag in ["code", "tt"] and not self.pre: self.o('`') #TODO: `` `this` ``
         if tag == "abbr":
             if start:
                 self.abbr_title = None
@@ -491,10 +450,10 @@ class _html2text(HTMLParser.HTMLParser):
                     self.abbr_list[self.abbr_data] = self.abbr_title
                     self.abbr_title = None
                 self.abbr_data = ''
-        
-        if tag == "a" and not IGNORE_ANCHORS:
+
+        if tag == "a" and not self.ignore_links:
             if start:
-                if has_key(attrs, 'href') and not (SKIP_INTERNAL_LINKS and attrs['href'].startswith('#')): 
+                if has_key(attrs, 'href') and not (self.skip_internal_links and attrs['href'].startswith('#')):
                     self.astack.append(attrs)
                     self.o("[")
                 else:
@@ -503,7 +462,7 @@ class _html2text(HTMLParser.HTMLParser):
                 if self.astack:
                     a = self.astack.pop()
                     if a:
-                        if INLINE_LINKS:
+                        if self.inline_links:
                             self.o("](" + a['href'] + ")")
                         else:
                             i = self.previousIndex(a)
@@ -515,12 +474,12 @@ class _html2text(HTMLParser.HTMLParser):
                                 a['outcount'] = self.outcount
                                 self.a.append(a)
                             self.o("][" + str(a['count']) + "]")
-        
-        if tag == "img" and start and not IGNORE_IMAGES:
+
+        if tag == "img" and start and not self.ignore_images:
             if has_key(attrs, 'src'):
                 attrs['href'] = attrs['src']
                 alt = attrs.get('alt', '')
-                if INLINE_LINKS:
+                if self.inline_links:
                     self.o("![")
                     self.o(alt)
                     self.o("]("+ attrs['href'] +")")
@@ -536,18 +495,18 @@ class _html2text(HTMLParser.HTMLParser):
                     self.o("![")
                     self.o(alt)
                     self.o("]["+ str(attrs['count']) +"]")
-        
+
         if tag == 'dl' and start: self.p()
         if tag == 'dt' and not start: self.pbr()
         if tag == 'dd' and start: self.o('    ')
         if tag == 'dd' and not start: self.pbr()
-        
+
         if tag in ["ol", "ul"]:
             # Google Docs create sub lists as top level lists
             if (not self.list) and (not self.lastWasList):
                 self.p()
             if start:
-                if options.google_doc:
+                if self.google_doc:
                     list_style = google_list_style(tag_style)
                 else:
                     list_style = tag
@@ -558,26 +517,26 @@ class _html2text(HTMLParser.HTMLParser):
             self.lastWasList = True
         else:
             self.lastWasList = False
-        
+
         if tag == 'li':
             self.pbr()
             if start:
                 if self.list: li = self.list[-1]
                 else: li = {'name':'ul', 'num':0}
-                if options.google_doc:
-                    nest_count = google_nest_count(tag_style)
+                if self.google_doc:
+                    nest_count = self.google_nest_count(tag_style)
                 else:
                     nest_count = len(self.list)
                 self.o("  " * nest_count) #TODO: line up <ol><li>s > 9 correctly.
-                if li['name'] == "ul": self.o(options.ul_item_mark + " ")
+                if li['name'] == "ul": self.o(self.ul_item_mark + " ")
                 elif li['name'] == "ol":
                     li['num'] += 1
                     self.o(str(li['num'])+". ")
                 self.start = 1
-        
+
         if tag in ["table", "tr"] and start: self.p()
         if tag == 'td': self.pbr()
-        
+
         if tag == "pre":
             if start:
                 self.startpre = 1
@@ -585,7 +544,7 @@ class _html2text(HTMLParser.HTMLParser):
             else:
                 self.pre = 0
             self.p()
-            
+
     def pbr(self):
         if self.p_p == 0: self.p_p = 1
 
@@ -594,19 +553,19 @@ class _html2text(HTMLParser.HTMLParser):
     def soft_br(self):
         self.pbr()
         self.br_toggle = '  '
-    
+
     def o(self, data, puredata=0, force=0):
         if self.abbr_data is not None: self.abbr_data += data
-        
-        if not self.quiet: 
-            if options.google_doc:
+
+        if not self.quiet:
+            if self.google_doc:
                 # prevent white space immediately after 'begin emphasis' marks ('**' and '_')
                 lstripped_data = data.lstrip()
                 if self.drop_white_space and not (self.pre or self.code):
                     data = lstripped_data
                 if lstripped_data != '':
                     self.drop_white_space = 0
-            
+
             if puredata and not self.pre:
                 data = re.sub('\s+', ' ', data)
                 if data and data[0] == ' ':
@@ -617,14 +576,14 @@ class _html2text(HTMLParser.HTMLParser):
             if self.startpre:
                 #self.out(" :") #TODO: not output when already one there
                 self.startpre = 0
-            
+
             bq = (">" * self.blockquote)
             if not (force and data and data[0] == ">") and self.blockquote: bq += " "
-            
+
             if self.pre:
                 bq += "    "
                 data = data.replace("\n", "\n"+bq)
-            
+
             if self.start:
                 self.space = 0
                 self.p_p = 0
@@ -640,18 +599,18 @@ class _html2text(HTMLParser.HTMLParser):
                 self.out((self.br_toggle+'\n'+bq)*self.p_p)
                 self.space = 0
                 self.br_toggle = ''
-                
+
             if self.space:
                 if not self.lastWasNL: self.out(' ')
                 self.space = 0
 
-            if self.a and ((self.p_p == 2 and LINKS_EACH_PARAGRAPH) or force == "end"):
+            if self.a and ((self.p_p == 2 and self.links_each_paragraph) or force == "end"):
                 if force == "end": self.out("\n")
 
                 newa = []
                 for link in self.a:
                     if self.outcount > link['outcount']:
-                        self.out("   ["+ str(link['count']) +"]: " + urlparse.urljoin(self.baseurl, link['href'])) 
+                        self.out("   ["+ str(link['count']) +"]: " + urlparse.urljoin(self.baseurl, link['href']))
                         if has_key(link, 'title'): self.out(" ("+link['title']+")")
                         self.out("\n")
                     else:
@@ -660,7 +619,7 @@ class _html2text(HTMLParser.HTMLParser):
                 if self.a != newa: self.out("\n") # Don't need an extra line when nothing was done.
 
                 self.a = newa
-            
+
             if self.abbr_list and force == "end":
                 for abbr, definition in self.abbr_list.items():
                     self.out("  *[" + abbr + "]: " + definition + "\n")
@@ -676,9 +635,78 @@ class _html2text(HTMLParser.HTMLParser):
             self.style_def.update(dumb_css_parser(data))
 
         self.o(data, 1)
-    
+
     def unknown_decl(self, data): pass
 
+    def charref(self, name):
+        if name[0] in ['x','X']:
+            c = int(name[1:], 16)
+        else:
+            c = int(name)
+
+        if not self.unicode_snob and c in unifiable_n.keys():
+            return unifiable_n[c]
+        else:
+            try:
+                return unichr(c)
+            except NameError: #Python3
+                return chr(c)
+
+    def entityref(self, c):
+        if not self.unicode_snob and c in unifiable.keys():
+            return unifiable[c]
+        else:
+            try: name2cp(c)
+            except KeyError: return "&" + c + ';'
+            else:
+                try:
+                    return unichr(name2cp(c))
+                except NameError: #Python3
+                    return chr(name2cp(c))
+
+    def replaceEntities(self, s):
+        s = s.group(1)
+        if s[0] == "#":
+            return self.charref(s[1:])
+        else: return self.entityref(s)
+
+    r_unescape = re.compile(r"&(#?[xX]?(?:[0-9a-fA-F]+|\w{1,8}));")
+    def unescape(self, s):
+        return self.r_unescape.sub(self.replaceEntities, s)
+
+    def google_nest_count(self, style):
+        """calculate the nesting count of google doc lists"""
+        nest_count = 0
+        if 'margin-left' in style:
+            nest_count = int(style['margin-left'][:-2]) / self.google_list_indent
+        return nest_count
+
+
+    def optwrap(self, text):
+        """Wrap all paragraphs in the provided text."""
+        if not self.body_width:
+            return text
+
+        assert wrap, "Requires Python 2.3."
+        result = ''
+        newlines = 0
+        for para in text.split("\n"):
+            if len(para) > 0:
+                if para[0] != ' ' and para[0] != '-' and para[0] != '*':
+                    for line in wrap(para, self.body_width):
+                        result += line + "\n"
+                    result += "\n"
+                    newlines = 2
+                else:
+                    if not onlywhite(para):
+                        result += para + "\n"
+                        newlines = 1
+            else:
+                if newlines < 2:
+                    result += "\n"
+                    newlines += 1
+        return result
+
 def wrapwrite(text):
     text = text.encode('utf-8')
     try: #Python3
@@ -686,50 +714,42 @@ def wrapwrite(text):
     except AttributeError:
         sys.stdout.write(text)
 
-def html2text_file(html, out=wrapwrite, baseurl=''):
-    h = _html2text(out, baseurl)
-    h.feed(html)
-    h.feed("")
-    return h.close()
-
 def html2text(html, baseurl=''):
-    return optwrap(html2text_file(html, None, baseurl))
+    h = HTML2Text(baseurl=baseurl)
+    return h.handle(html)
 
-class Storage: pass
-options = Storage()
-options.google_doc = False
-options.ul_item_mark = '*'
+def unescape(s, unicode_snob=False):
+    h = HTML2Text()
+    h.unicode_snob = unicode_snob
+    return h.unescape(s)
 
-if __name__ == "__main__":
+def main():
     baseurl = ''
-    
+
     p = optparse.OptionParser('%prog [(filename|url) [encoding]]',
                               version='%prog ' + __version__)
+    p.add_option("--ignore-emphasis", dest="ignore_emphasis", action="store_true", 
+        default=IGNORE_EMPHASIS, help="don't include any formatting for emphasis")
+    p.add_option("--ignore-links", dest="ignore_links", action="store_true",
+        default=IGNORE_ANCHORS, help="don't include any formatting for links")
+    p.add_option("--ignore-images", dest="ignore_images", action="store_true",
+        default=IGNORE_IMAGES, help="don't include any formatting for images")
     p.add_option("-g", "--google-doc", action="store_true", dest="google_doc",
         default=False, help="convert an html-exported Google Document")
     p.add_option("-d", "--dash-unordered-list", action="store_true", dest="ul_style_dash",
         default=False, help="use a dash rather than a star for unordered list items")
     p.add_option("-b", "--body-width", dest="body_width", action="store", type="int",
-        default=78, help="number of characters per output line, 0 for no wrap")
+        default=BODY_WIDTH, help="number of characters per output line, 0 for no wrap")
     p.add_option("-i", "--google-list-indent", dest="list_indent", action="store", type="int",
         default=GOOGLE_LIST_INDENT, help="number of pixels Google indents nested lists")
     p.add_option("-s", "--hide-strikethrough", action="store_true", dest="hide_strikethrough",
         default=False, help="hide strike-through text. only relevent when -g is specified as well")
     (options, args) = p.parse_args()
 
-    # handle options
-    if options.ul_style_dash:
-        options.ul_item_mark = '-'
-    else:
-        options.ul_item_mark = '*'
-
-    BODY_WIDTH = options.body_width
-    GOOGLE_LIST_INDENT = options.list_indent
-
     # process input
+    encoding = "utf-8"
     if len(args) > 0:
         file_ = args[0]
-        encoding = None
         if len(args) == 2:
             encoding = args[1]
         if len(args) > 2:
@@ -738,17 +758,15 @@ if __name__ == "__main__":
         if file_.startswith('http://') or file_.startswith('https://'):
             baseurl = file_
             j = urllib.urlopen(baseurl)
-            text = j.read()
+            data = j.read()
             if encoding is None:
                 try:
                     from feedparser import _getCharacterEncoding as enc
                 except ImportError:
                     enc = lambda x, y: ('utf-8', 1)
-                encoding = enc(j.headers, text)[0]
+                encoding = enc(j.headers, data)[0]
                 if encoding == 'us-ascii':
                     encoding = 'utf-8'
-            data = text.decode(encoding)
-
         else:
             data = open(file_, 'rb').read()
             if encoding is None:
@@ -757,7 +775,24 @@ if __name__ == "__main__":
                 except ImportError:
                     detect = lambda x: {'encoding': 'utf-8'}
                 encoding = detect(data)['encoding']
-            data = data.decode(encoding)
     else:
         data = sys.stdin.read()
-    wrapwrite(html2text(data, baseurl))
+
+    data = data.decode(encoding)
+    h = HTML2Text(baseurl=baseurl)
+    # handle options
+    if options.ul_style_dash: h.ul_item_mark = '-'
+
+    h.body_width = options.body_width
+    h.list_indent = options.list_indent
+    h.ignore_emphasis = options.ignore_emphasis
+    h.ignore_links = options.ignore_links
+    h.ignore_images = options.ignore_images
+    h.google_doc = options.google_doc
+    h.hide_strikethrough = options.hide_strikethrough
+
+    wrapwrite(h.handle(data))
+
+
+if __name__ == "__main__":
+    main()
